﻿' *****************************************************************************
' NAntRunner
' A simple, standalone, utility for running NAnt build scripts.
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' *****************************************************************************
Imports System.IO
Imports NRBusinessServices

Public Class NAntRunner

    Private _PreviousScriptIndex As Integer
    Private _NAntRunnerSettings As NAntRunnerSettings


    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Me._NAntRunnerSettings = New NAntRunnerSettings()

        Me.MainNavigationTree.AddGroupMethod = AddressOf Me.AddGroup
        Me.MainNavigationTree.AddNodeMethod = AddressOf Me.AddScript
    End Sub


#Region " Properties "
    ''' <summary>
    ''' This property returns the value entered into the Additional Arguments text box.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Property NAntExecutablePath() As String
        Get
            ' TODO: This should check for the existence of the executable
            Return Me.NAntExecutableTextBox.Text.Trim
        End Get
        Set(ByVal value As String)
            If (IsNothing(value)) Then value = ""
            Me.NAntExecutableTextBox.Text = value
        End Set
    End Property




    ''' <summary>
    ''' This will return a NavigationTreeNode object for the current tree node if it is a script; it will
    ''' return Nothing otherwise.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property CurrentScriptNode() As NavigationTreeNode
        Get
            If (IsNothing(Me.MainNavigationTree.SelectedItem)) Then Return Nothing
            If (CType(Me.MainNavigationTree.SelectedItem, NavigationTreeNode).Level = 0) Then Return Nothing
            Return CType(Me.MainNavigationTree.SelectedItem, NavigationTreeNode)
        End Get
    End Property


    ''' <summary>
    ''' This will return a NavigationTreeNode object for the current tree node if it is a script group, and the
    ''' current node's parent if the current node is a script.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property CurrentScriptGroupNode() As NavigationTreeNode
        Get
            If (IsNothing(Me.MainNavigationTree.SelectedItem)) Then Return Nothing
            If (CType(Me.MainNavigationTree.SelectedItem, NavigationTreeNode).Level = 1) Then Return CType(Me.MainNavigationTree.SelectedItem.Parent, NavigationTreeNode)
            Return CType(Me.MainNavigationTree.SelectedItem, NavigationTreeNode)
        End Get
    End Property


    ''' <summary>
    ''' This will return the currently selected NAntScript object, or Nothing if one isn't selected.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property CurrentScript() As NAntScript
        Get
            If (Not IsNothing(Me.CurrentScriptNode)) Then Return New NAntScript(Me.CurrentScriptNode.Path)
            Return Nothing
        End Get
    End Property



    ''' <summary>
    ''' This returns True if a NAntScript node has been selected from the library, and False otherwise
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property IsScriptSelected() As Boolean
        Get
            Return Not IsNothing(Me.CurrentScriptNode)
        End Get
    End Property


    ''' <summary>
    ''' This returns True if a Script Group node has been selected from the library, and False otherwise
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property IsScriptGroupSelected() As Boolean
        Get
            Return Not IsNothing(Me.CurrentScriptGroupNode)
        End Get
    End Property


    ''' <summary>
    ''' This property returns the value of the selected target framework, or an empty string if none is selected.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property CurrentTargetFramework() As String
        Get
            If (Me.FrameworkDropDownBox.Items.Count = 0) Then Return ""
            If (IsNothing(Me.FrameworkDropDownBox.SelectedItem)) Then Return ""
            Return CType(Me.FrameworkDropDownBox.SelectedItem, ListItem).Value
        End Get
    End Property

    ''' <summary>
    ''' This property returns the value entered into the Additional Arguments text box.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property CurrentOtherArgs() As String
        Get
            Return Me.AdditionalNAntArgsTextbox.Text.Trim
        End Get
    End Property

    ''' <summary>
    ''' This property returns the value of the selected build target, or an empty string if none is selected.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property CurrentBuildTarget() As String
        Get
            If (Me.BuildTargetDropdown.Items.Count = 0) Then Return ""
            If (IsNothing(Me.BuildTargetDropdown.SelectedItem)) Then Return ""
            Return CType(Me.BuildTargetDropdown.SelectedItem, ListItem).Value
        End Get
    End Property
#End Region


#Region " Private Methods "
    ''' <summary>
    ''' This runs the script selected
    ''' </summary>
    ''' <remarks>
    ''' The original code for this routine came from http://www.thescarms.com/dotnet/Process.aspx.
    ''' </remarks>
    Private Sub RunScript()
        Dim MyNAntProcess As NAntProcess
        Dim OutputString, ErrorString As String

        If (Not Me.IsScriptSelected) Then Return

        ' Run the script
        MyNAntProcess = New NAntProcess(Me.NAntExecutablePath, _
                                        Me.CurrentScript.ScriptPath, _
                                        Me.CurrentTargetFramework, _
                                        Me.CurrentBuildTarget, _
                                        Me.EnableVerboseMessages.Checked, _
                                        Me.CurrentOtherArgs)
        MyNAntProcess.Start()

        ' Grab the results
        OutputString = MyNAntProcess.StandardOutput
        ErrorString = MyNAntProcess.StandardError

        ' Clean up
        ' TODO: The close method should preserve the StandardOutput and StandardError property values from the process so that those values are available after this call.
        ' TODO: Create unit tests around this.
        MyNAntProcess.Close()
        MyNAntProcess = Nothing

        ' Display the results
        Me.ResultsBox.Text = OutputString
        If (ErrorString.Trim.Length > 0) Then Me.ResultsBox.AppendText(vbCrLf & vbCrLf & "-- E R R O R --" & vbCrLf & ErrorString)
    End Sub


    ''' <summary>
    ''' This saves the user's settings to the local "settings" file.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub SaveSettings()
        Dim CurrentScriptGroupNode, CurrentScriptNode As NavigationTreeNode
        Dim NewScriptGroup As NAntScriptGroup
        Dim i As Integer

        Me._NAntRunnerSettings.NAntExecutablePath = Me.NAntExecutablePath
        Me._NAntRunnerSettings.UtilitySize = New Rectangle(0, 0, Me.Width, Me.Height)

        Me._NAntRunnerSettings.ScriptGroups.Clear()
        For Each CurrentScriptGroupNode In Me.MainNavigationTree.Items
            NewScriptGroup = New NAntScriptGroup(CurrentScriptGroupNode.DisplayName)

            If (CurrentScriptGroupNode.Nodes.Count > 0) Then
                For i = 0 To CurrentScriptGroupNode.Nodes.Count - 1
                    ' TODO: The .Path and .DisplayName properties are both empty here
                    CurrentScriptNode = CType(CurrentScriptGroupNode.Nodes(i), NavigationTreeNode)
                    NewScriptGroup.Scripts.Add(New NAntScript(CurrentScriptNode.Path, CurrentScriptNode.DisplayName))
                Next
                'For Each CurrentScriptNode In CurrentScriptGroupNode.Nodes
                '    NewScriptGroup.Scripts.Add(New NAntScript(CurrentScriptNode.Path, CurrentScriptNode.DisplayName))
                'Next
            End If

            Me._NAntRunnerSettings.ScriptGroups.Add(NewScriptGroup)
        Next

        Me._NAntRunnerSettings.Save()

        Me.Message.Visible = True
        Me.Message.Text = "Settings Last Saved: " & Now.ToLongTimeString
    End Sub


    ''' <summary>
    ''' This loads up the interface with the values from the settings file, if it exists.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadSettings()
        Dim NewGroupNode, NewScriptNode As NavigationTreeNode
        Dim CurrentScriptGroup As NAntScriptGroup
        Dim CurrentScript As NAntScript

        Me.NAntExecutablePath = Me._NAntRunnerSettings.NAntExecutablePath
        Me.PopulateTargetFrameworks()

        ' Populate the list of script groups
        Dim ScriptGroups = From sg In Me._NAntRunnerSettings.ScriptGroups Select sg
        For Each CurrentScriptGroup In ScriptGroups
            NewGroupNode = New NavigationTreeNode(CurrentScriptGroup.Name)

            For Each CurrentScript In CurrentScriptGroup.Scripts
                NewScriptNode = New NavigationTreeNode(CurrentScript.ScriptName, CurrentScript.ScriptPath)
                NewGroupNode.Nodes.Add(NewScriptNode)
            Next

            Me.MainNavigationTree.Items.Add(NewGroupNode)
        Next

        ' TODO: These should be verified as being valid numbers, and greater than the form minimums
        Me.Width = Me._NAntRunnerSettings.UtilitySize.Width
        Me.Height = Me._NAntRunnerSettings.UtilitySize.Height
    End Sub


    ''' <summary>
    ''' This takes care of enabling/disabling controls that depend on a value being selected from the script library list box.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateUIWhenScriptIsSelected()
        Me.PathToScriptTextbox.Text = ""
        If (Me.IsScriptSelected) Then Me.PathToScriptTextbox.Text = Me.CurrentScript.ScriptPath

        ' Script detail controls
        Me.FrameworkDropDownBox.Enabled = Me.IsScriptSelected
        Me.BuildTargetDropdown.Enabled = Me.IsScriptSelected
        Try
            If (Me.IsScriptSelected) Then Me.PopulateBuildTargets()
        Catch ex As Exception
            Me.ResultsBox.AppendText(vbCrLf & vbCrLf & "-- E R R O R --" & vbCrLf & ex.Message)
        End Try
        Me.AdditionalNAntArgsTextbox.Enabled = Me.IsScriptSelected

        Me.RunScriptButton.Enabled = Me.IsScriptSelected
    End Sub


    ''' <summary>
    ''' This updates the Version information in a label in the footer; it pulls this via reflection from the executing assembly
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub LoadVersionInfo()
        Dim asnCurrent As System.Reflection.AssemblyName

        asnCurrent = System.Reflection.Assembly.GetExecutingAssembly.GetName
        Me.VersionLabel.Text = asnCurrent.Version.Major & "." & _
                                asnCurrent.Version.Minor.ToString.PadLeft(1, "0") & "." & _
                                asnCurrent.Version.Build.ToString.PadLeft(1, "0") & vbCrLf
    End Sub


    ''' <summary>
    ''' This populates the Target Framework drop down with the list available in the NAnt configuration file.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub PopulateTargetFrameworks()
        Dim NAntConfigFile As String
        Dim NAntConfigFileXML As XDocument
        Dim Target As XElement

        Me.FrameworkDropDownBox.Items.Clear()
        Me.FrameworkDropDownBox.Items.Add(New ListItem("{Default}", ""))
        If (File.Exists(Me.NAntExecutablePath)) Then
            NAntConfigFile = Me.NAntExecutablePath & ".config"
            If (File.Exists(NAntConfigFile)) Then
                NAntConfigFileXML = XDocument.Load(NAntConfigFile)

                ' TODO: The <framework> elements are contained in <platform> elements that have "win32" and "unix".  This loop lumps these all together; they should be broken out
                Dim Targets = From t In NAntConfigFileXML...<framework> Select t

                For Each Target In Targets
                    Me.FrameworkDropDownBox.Items.Add(New ListItem(Target.@name.ToString & " (" & Target.@description.ToString & "; CLR v" & Target.@clrversion.ToString & ")", Target.@name.ToString))
                Next
            End If
        End If

        Me.FrameworkDropDownBox.SelectedIndex = 0
    End Sub


    ''' <summary>
    ''' This populates the Build Targets drop down with the list available in the build script selected.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub PopulateBuildTargets()
        Dim BuildScriptXML As XDocument
        Dim BuildTarget As XElement

        Me.BuildTargetDropdown.Items.Clear()
        Me.BuildTargetDropdown.Items.Add(New ListItem("{Default}", ""))

        If (File.Exists(Me.CurrentScript.ScriptPath)) Then
            BuildScriptXML = XDocument.Load(Me.CurrentScript.ScriptPath)

            Dim BuildTargets = From bt In BuildScriptXML...<target> Select bt
            For Each BuildTarget In BuildTargets
                Me.BuildTargetDropdown.Items.Add(New ListItem(BuildTarget.@name, BuildTarget.@name))
            Next
        End If

        Me.BuildTargetDropdown.SelectedIndex = 0
    End Sub


    ''' <summary>
    ''' This removes the currently selected node from the tree.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RemoveNode()
        Dim CurrentNodeText As String
        If (Me.IsScriptSelected Or Me.IsScriptGroupSelected) Then
            CurrentNodeText = Me.MainNavigationTree.SelectedItem.DisplayName
            If (Me.IsScriptSelected) Then
                If (MsgBox("Do you really want to remove '" & CurrentNodeText & "'?", MsgBoxStyle.YesNo) = MsgBoxResult.No) Then Return
            Else
                If (MsgBox("This will remove the '" & CurrentNodeText & "' group, and all scripts contained in it.  Do you really want to do this?", MsgBoxStyle.YesNo) = MsgBoxResult.No) Then Return
            End If

            Me.MainNavigationTree.SelectedItem.Remove()
            Me.SaveSettings()
        End If
    End Sub


    ''' <summary>
    ''' This adds a new script group to the end of the list.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AddGroup()
        Me.MainNavigationTree.Items.Add(New NavigationTreeNode("New Group"))
        Me.MainNavigationTree.SelectedItem = Me.MainNavigationTree.Items(Me.MainNavigationTree.Items.Count - 1)
        Me.MainNavigationTree.SelectedItem.BeginEdit()
    End Sub


    ''' <summary>
    ''' This opens a dialog that allows the user to select a build script to add to the list.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub AddScript()
        Dim OpenScriptDialog As System.Windows.Forms.OpenFileDialog
        Dim FileSelected As String
        Dim NewScriptNode As NavigationTreeNode
        Dim NewNAntScript As NAntScript

        OpenScriptDialog = New System.Windows.Forms.OpenFileDialog
        OpenScriptDialog.Filter = "All Files|*.*"
        OpenScriptDialog.Title = "Select NAnt Script"

        If (OpenScriptDialog.ShowDialog = Windows.Forms.DialogResult.OK) Then
            FileSelected = OpenScriptDialog.FileName
            NewNAntScript = New NAntScript(FileSelected)
            NewScriptNode = New NavigationTreeNode(NewNAntScript.ScriptName, NewNAntScript.ScriptPath)

            If (Me.IsScriptSelected) Then
                Me.CurrentScriptNode.Parent.Nodes.Add(NewScriptNode)
                Me.MainNavigationTree.SelectedItem = Me.CurrentScriptNode.Parent.LastNode
            Else
                Me.CurrentScriptGroupNode.Nodes.Add(NewScriptNode)
                Me.MainNavigationTree.SelectedItem = Me.CurrentScriptGroupNode.LastNode
            End If
            Me.SaveSettings()
        End If

        OpenScriptDialog.Dispose()
        OpenScriptDialog = Nothing
    End Sub
#End Region



#Region " Event Handlers "

    ''' <summary>
    ''' This initializes the form elements
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub NAntRunner_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me._PreviousScriptIndex = -1
        Me.LoadSettings()
        Me.UpdateUIWhenScriptIsSelected()
        Me.LoadVersionInfo()
        Me.MainNavigationTree.ExpandAll()
    End Sub


    ''' <summary>
    ''' This adjusts the ScriptList list box's width by the same amount that the main window's Width changed.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub NAntRunner_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        Me.SaveSettings()
    End Sub


    ''' <summary>
    ''' This provides the user a way to browse for the NAnt executable.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub BrowseToNAntButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BrowseToNAntButton.Click
        Dim FindNAntExecutableDialog As System.Windows.Forms.OpenFileDialog

        FindNAntExecutableDialog = New System.Windows.Forms.OpenFileDialog
        FindNAntExecutableDialog.Filter = "Executables|*.exe|All Files|*.*"
        FindNAntExecutableDialog.Title = "Select NAnt Executable"

        If (FindNAntExecutableDialog.ShowDialog = Windows.Forms.DialogResult.OK) Then
            Me.NAntExecutablePath = FindNAntExecutableDialog.FileName
            Me.MainNavigationTree.Focus()
            Me.SaveSettings()
            Me.PopulateTargetFrameworks()
        End If

        FindNAntExecutableDialog.Dispose()
        FindNAntExecutableDialog = Nothing
    End Sub


    ''' <summary>
    ''' This looks for the key combination CTRL-A, and if found selects all of the text from the ResultsBox.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub ResultsBox_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles ResultsBox.KeyPress
        If e.KeyChar = Convert.ToChar(1) Then
            DirectCast(sender, TextBox).SelectAll()
            e.Handled = True
        End If
    End Sub


    ''' <summary>
    ''' This closes the utility.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub CloseButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CloseButton.Click
        End
    End Sub


    ''' <summary>
    ''' This runs the script requested.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainNavigationTree_ItemExecuted(ByVal e As System.EventArgs) Handles MainNavigationTree.ItemExecuted
        If (IsNothing(Me.MainNavigationTree.SelectedItem)) Then Return

        Try
            Me.Cursor = Cursors.WaitCursor
            Me.RunScript()
        Catch ex As Exception
            Me.ResultsBox.AppendText(ex.Message)
        Finally
            Me.Cursor = Cursors.Default
        End Try

    End Sub


    ''' <summary>
    ''' This handles when an item has been updated.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainNavigationTree_ItemsEdited(ByVal e As System.EventArgs) Handles MainNavigationTree.ItemsEdited
        Me.SaveSettings()
    End Sub


    ''' <summary>
    ''' This handles when an item has been selected.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainNavigationTree_ItemSelected(ByVal e As System.EventArgs) Handles MainNavigationTree.ItemSelected
        Me.UpdateUIWhenScriptIsSelected()
    End Sub


    ''' <summary>
    ''' This handles when a problem occurs trying to edit an item.
    ''' </summary>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainNavigationTree_ModificationExceptionThrown(ByVal e As NavigationTree.ModificationExceptionEventArgs) Handles MainNavigationTree.ModificationExceptionThrown
        Me.ResultsBox.AppendText(e.ExceptionMessage)
    End Sub


    ''' <summary>
    ''' This runs the script selected, if there is one.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub RunScriptButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RunScriptButton.Click
        Me.Cursor = Cursors.WaitCursor
        Me.RunScript()
        Me.Cursor = Cursors.Default
    End Sub
#End Region


End Class
