﻿' *****************************************************************************
' NAntRunner
' A simple, standalone, utility for running NAnt build scripts.
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' *****************************************************************************
Imports System.Text

Public Class NAntProcess

    Private _NAntExecutable As String
    Private _ScriptToRun As String
    Private _SystemProcess As Process
    Private _TargetFramework As String
    Private _BuildTarget As String
    Private _OtherArgs As String

#Region " Exceptions "
    Public Class ExecutableMissingException
        Inherits Exception
    End Class

    Public Class ScriptToRunMissingException
        Inherits Exception
    End Class
#End Region


#Region " Constructors "

    ''' <summary>
    ''' The minimal constructor
    ''' </summary>
    ''' <param name="NAntExecutablePath">The full path and file name to the NAnt executable.</param>
    ''' <param name="NewScriptToRun">The full path and file name to the NAnt script to run.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NAntExecutablePath As String, _
                   ByVal NewScriptToRun As String)
        Me.New(NAntExecutablePath, NewScriptToRun, "", "", "")
    End Sub


    ''' <summary>
    ''' One of the "intermediary" constructors, which allows the user to specify only the NAnt executable path, script to be run, and the framework to target.
    ''' </summary>
    ''' <param name="NAntExecutablePath">The full path and file name to the NAnt executable.</param>
    ''' <param name="NewScriptToRun">The full path and file name to the NAnt script to run.</param>
    ''' <param name="NewTargetFramework">The framework to target</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NAntExecutablePath As String, _
                   ByVal NewScriptToRun As String, _
                   ByVal NewTargetFramework As String)
        Me.New(NAntExecutablePath, NewScriptToRun, NewTargetFramework, "", "")
    End Sub


    ''' <summary>
    ''' One of the "intermediary" constructors, which allows the user to specify the NAnt executable path, script to be run, the target within the script, and the framework to target.
    ''' </summary>
    ''' <param name="NAntExecutablePath">The full path and file name to the NAnt executable.</param>
    ''' <param name="NewScriptToRun">The full path and file name to the NAnt script to run.</param>
    ''' <param name="NewTargetFramework">The framework to target</param>
    ''' <param name="NewBuildTarget">The target within the build script to run.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NAntExecutablePath As String, _
                   ByVal NewScriptToRun As String, _
                   ByVal NewTargetFramework As String, _
                   ByVal NewBuildTarget As String)
        Me.New(NAntExecutablePath, NewScriptToRun, NewTargetFramework, NewBuildTarget, "")
    End Sub


    ''' <summary>
    ''' The complete constructor, which takes all possible values
    ''' </summary>
    ''' <param name="NAntExecutablePath">The full path and file name to the NAnt executable.</param>
    ''' <param name="NewScriptToRun">The full path and file name to the NAnt script to run.</param>
    ''' <param name="NewTargetFramework">The framework to target</param>
    ''' <param name="NewBuildTarget">The target within the build script to run.</param>
    ''' <param name="NewOtherArgs">Any other argument to be passed to NAnt</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal NAntExecutablePath As String, _
                   ByVal NewScriptToRun As String, _
                   ByVal NewTargetFramework As String, _
                   ByVal NewBuildTarget As String, _
                   ByVal NewOtherArgs As String)

        If (IsNothing(NAntExecutablePath)) Then Throw New ExecutableMissingException
        If (NAntExecutablePath.Trim.Length = 0) Then Throw New ExecutableMissingException
        Me._NAntExecutable = NAntExecutablePath

        If (IsNothing(NewScriptToRun)) Then Throw New ScriptToRunMissingException
        If (NewScriptToRun.Trim.Length = 0) Then Throw New ScriptToRunMissingException
        Me._ScriptToRun = NewScriptToRun

        Me._TargetFramework = NewTargetFramework
        Me._BuildTarget = NewBuildTarget
        Me._OtherArgs = NewOtherArgs

        Me._SystemProcess = New Process
        Me._SystemProcess.StartInfo.FileName = Me._NAntExecutable
        Me._SystemProcess.StartInfo.UseShellExecute = False
        Me._SystemProcess.StartInfo.CreateNoWindow = True
        Me._SystemProcess.StartInfo.RedirectStandardOutput = True
        Me._SystemProcess.StartInfo.RedirectStandardError = True
        Me._SystemProcess.StartInfo.Arguments = Me.GetArgumentsForNAnt()
    End Sub
#End Region






#Region " Properties "
    ''' <summary>
    ''' This exposes the path to the NAnt executable
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' While it is unlikely to change, this property is read/write to allow the path
    ''' to NAnt to change without having to destroy and recreate the object.
    ''' </remarks>
    Public Property NAntExecutable() As String
        Get
            If (IsNothing(Me._NAntExecutable)) Then Return ""
            Return Me._NAntExecutable.Trim
        End Get
        Set(ByVal value As String)
            ' TODO: Need to valid the path entered
            ' TODO: Need unit tests around setting this value
            Me._NAntExecutable = value
        End Set
    End Property


    ''' <summary>
    ''' This exposes the path to the NAnt script to run
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks>
    ''' This property is read/write to allow the calling code to use the same object to
    ''' run multiple different scripts.
    ''' </remarks>
    Public Property ScriptToRun() As String
        Get
            If (IsNothing(Me._ScriptToRun)) Then Return ""
            Return Me._ScriptToRun.Trim
        End Get
        Set(ByVal value As String)
            ' TODO: Need to valid the path entered
            ' TODO: Need unit tests around setting this value
            Me._ScriptToRun = value
        End Set
    End Property


    ''' <summary>
    ''' This exposes the target framework setting
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property TargetFramework() As String
        Get
            If (IsNothing(Me._TargetFramework)) Then Return ""
            Return Me._TargetFramework.Trim
        End Get
        Set(ByVal value As String)
            ' TODO: Need to valid the value entered
            ' TODO: Need unit tests around setting this value
            Me._TargetFramework = value
        End Set
    End Property


    ''' <summary>
    ''' This exposes the other arguments setting
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property OtherArgs() As String
        Get
            If (IsNothing(Me._OtherArgs)) Then Return ""
            Return Me._OtherArgs.Trim
        End Get
        Set(ByVal value As String)
            ' TODO: Need to valid the value entered
            ' TODO: Need unit tests around setting this value
            Me._OtherArgs = value
        End Set
    End Property


    ''' <summary>
    ''' This exposes the build target setting
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property BuildTarget() As String
        Get
            If (IsNothing(Me._BuildTarget)) Then Return ""
            Return Me._BuildTarget.Trim
        End Get
        Set(ByVal value As String)
            ' TODO: Need to valid the value entered
            ' TODO: Need unit tests around setting this value
            Me._BuildTarget = value
        End Set
    End Property


    ''' <summary>
    ''' This property exposes the StandardOutput value of the system process that is spawned by the constructor.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property StandardOutput() As String
        Get
            ' TODO: This needs to check that the output stream was redirected.
            ' TODO: This needs to check that Me._SystemProcess isn't Nothing
            ' TODO: This needs to check that Me._SystemProcess.StandardOutput isn't Nothing
            Return Me._NAntExecutable & " " & Me.GetArgumentsForNAnt & vbCrLf & vbCrLf & _
                    Me._SystemProcess.StandardOutput.ReadToEnd
        End Get
    End Property


    ''' <summary>
    ''' This property exposes the StandardError value of the system process that is spawned by the constructor.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property StandardError() As String
        Get
            ' TODO: This needs to check that the error stream was redirected.
            ' TODO: This needs to check that Me._SystemProcess isn't Nothing
            ' TODO: This needs to check that Me._SystemProcess.StandardOutput isn't Nothing
            Return Me._SystemProcess.StandardError.ReadToEnd
        End Get
    End Property


    ''' <summary>
    ''' This exposes the arguments used to call NAnt
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property BuildArguments() As String
        Get
            Return Me.GetArgumentsForNAnt
        End Get
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This starts the system process, and runs the script itself.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Start()
        ' TODO: Validate that Me._SystemProcess is really ready to go
        Me._SystemProcess.Start()
    End Sub


    ''' <summary>
    ''' This closes and disposes of the system process.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub Close()
        If Not Me._SystemProcess.HasExited Then Me._SystemProcess.Kill()
        Me._SystemProcess.Close()
        Me._SystemProcess = Nothing
    End Sub


    ''' <summary>
    ''' This compiles all of the parameters to pass to NAnt
    ''' </summary>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function GetArgumentsForNAnt() As String
        Dim ArgumentBuilder As StringBuilder

        ArgumentBuilder = New StringBuilder

        ArgumentBuilder.Append("-buildfile:""" & Me.ScriptToRun & """")
        If (Me.TargetFramework.Length > 0) Then ArgumentBuilder.Append(" -targetframework:" & Me.TargetFramework)
        If (Me.BuildTarget.Length > 0) Then ArgumentBuilder.Append(" " & Me.BuildTarget)
        If (Me.OtherArgs.Length > 0) Then ArgumentBuilder.Append(" " & Me.OtherArgs)

        Return ArgumentBuilder.ToString
    End Function
#End Region

End Class
