﻿' *****************************************************************************
' NAntRunner
' A simple, standalone, utility for running NAnt build scripts.
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' *****************************************************************************

Imports System.IO
Imports System.Collections
Imports System.Drawing

Public Class NAntRunnerSettings

    Public Const CURRENT_NANTRUNNER_SETTINGS_FILE_VERSION As Single = 0.3
    Private Const SETTINGS_FILE_NAME As String = "NAntRunnerSettings.xml"

    Private _NAntExecutablePath As String
    Private _UtilitySize As Rectangle
    Private _SettingsFileVersion As Single
    Private _ScriptGroups As List(Of NAntScriptGroup)


#Region " Properties"
    Public Property NAntExecutablePath() As String
        Get
            If (IsNothing(Me._NAntExecutablePath)) Then Me._NAntExecutablePath = ""
            Return Me._NAntExecutablePath
        End Get
        Set(ByVal value As String)
            Me._NAntExecutablePath = value
        End Set
    End Property


    Public Property UtilitySize() As Rectangle
        Get
            If (IsNothing(Me._UtilitySize)) Then Me._UtilitySize = New Rectangle(0, 0, 672, 546)
            Return Me._UtilitySize
        End Get
        Set(ByVal value As Rectangle)
            Me._UtilitySize = value
        End Set
    End Property


    Public Property ScriptGroups() As List(Of NAntScriptGroup)
        Get
            If (IsNothing(Me._ScriptGroups)) Then Me._ScriptGroups = New List(Of NAntScriptGroup)
            Return Me._ScriptGroups
        End Get
        Set(ByVal value As List(Of NAntScriptGroup))
            Me._ScriptGroups = value
        End Set
    End Property


    Public ReadOnly Property PathToSettingsFile() As String
        Get
            Dim ExecutionPath As String

            ExecutionPath = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location)

            Return Path.Combine(ExecutionPath, SETTINGS_FILE_NAME)
        End Get
    End Property


    Public ReadOnly Property SettingsFileVersion() As Single
        Get
            Return Me._SettingsFileVersion
        End Get
    End Property
#End Region


#Region " Constructors "
    Public Sub New()
        If (File.Exists(Me.PathToSettingsFile)) Then
            Me.Load(XDocument.Load(PathToSettingsFile))
        Else
            Me._NAntExecutablePath = ""
            Me._SettingsFileVersion = CURRENT_NANTRUNNER_SETTINGS_FILE_VERSION
            Me._ScriptGroups = New List(Of NAntScriptGroup)
        End If
    End Sub


    Public Sub New(ByVal SettingsFile As XDocument)
        Me.Load(SettingsFile)
    End Sub
#End Region


#Region " Methods "
    Private Sub Load(ByVal SettingsFile As XDocument)
        Dim NewScriptGroup As NAntScriptGroup

        If (IsNothing(SettingsFile)) Then Return

        Me._SettingsFileVersion = SettingsFile...<SettingsFileVersion>.Value

        Select Case Me._SettingsFileVersion
            Case 0.3
                Me._NAntExecutablePath = SettingsFile...<NAntExecutable>.Value.ToString
                Me._UtilitySize = New Rectangle(0, 0, SettingsFile...<UtilitySize>.<Width>.Value, SettingsFile...<UtilitySize>.<Height>.Value)

                Me._ScriptGroups = New List(Of NAntScriptGroup)
                Dim ScriptGroupsFromFile = From sg In SettingsFile...<ScriptGroup> Select sg
                For Each CurrentScriptGroup In ScriptGroupsFromFile
                    NewScriptGroup = New NAntScriptGroup(CurrentScriptGroup.@Name)

                    Dim ScriptsInGroup = From s In CurrentScriptGroup...<Script> Select s
                    For Each CurrentScript In ScriptsInGroup
                        NewScriptGroup.Scripts.Add(New NAntScript(CurrentScript...<ScriptPath>.Value, CurrentScript...<ScriptName>.Value))
                    Next

                    Me._ScriptGroups.Add(NewScriptGroup)
                Next

            Case Else
                ' Assume that this is a version 0.1 or 0.2 settings file
                Me._NAntExecutablePath = SettingsFile...<NAntExecutable>.Value.ToString
                Me._UtilitySize = New Rectangle(0, 0, SettingsFile...<UtilitySize>.<Width>.Value, SettingsFile...<UtilitySize>.<Height>.Value)

                Me._ScriptGroups = New List(Of NAntScriptGroup)
                Me._ScriptGroups.Add(New NAntScriptGroup("Default"))

                Dim ScriptsFromFile = From s In SettingsFile...<Script> Select s
                For Each NewScript In ScriptsFromFile
                    Me._ScriptGroups.Item(0).Scripts.Add(New NAntScript(NewScript...<ScriptPath>.Value))
                Next
        End Select

        ' Check to see if we just upgraded the file; if we did, save it out immediately
        If (Me._SettingsFileVersion < CURRENT_NANTRUNNER_SETTINGS_FILE_VERSION) Then
            Me._SettingsFileVersion = CURRENT_NANTRUNNER_SETTINGS_FILE_VERSION
            Me.Save()
        End If
    End Sub


    Public Function Save() As XDocument
        Dim SettingsFile As XDocument

        SettingsFile = <?xml version="1.0" encoding="UTF-16" standalone="yes"?>
                       <NAntRunnerSettings>
                           <SettingsFileVersion>
                               <%= Me._SettingsFileVersion.ToString.Trim %>
                           </SettingsFileVersion>
                           <NAntExecutable>
                               <%= Me.NAntExecutablePath %>
                           </NAntExecutable>
                           <UtilitySize>
                               <Width><%= Me.UtilitySize.Width %></Width>
                               <Height><%= Me.UtilitySize.Height %></Height>
                           </UtilitySize>
                           <ScriptGroups>
                               <%= From CurrentScriptGroup In Me.ScriptGroups Select <ScriptGroup Name=<%= CurrentScriptGroup.Name %>>
                                                                                         <%= From CurrentScript In CurrentScriptGroup.Scripts Select <Script>
                                                                                                                                                         <ScriptPath><%= CurrentScript.ScriptPath %></ScriptPath>
                                                                                                                                                         <ScriptName><%= CurrentScript.ScriptName %></ScriptName>
                                                                                                                                                     </Script> %>
                                                                                     </ScriptGroup> %>
                           </ScriptGroups>
                       </NAntRunnerSettings>

        SettingsFile.Save(Me.PathToSettingsFile)

        Return SettingsFile
    End Function
#End Region

End Class
