﻿' *****************************************************************************
' NAntRunner
' A simple, standalone, utility for running NAnt build scripts.
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' *****************************************************************************
Public Class NavigationTree


#Region " Events and Exceptions "
    Public Event ItemExecuted(ByVal e As EventArgs)
    Public Event ItemsEdited(ByVal e As EventArgs)
    Public Event ModificationExceptionThrown(ByVal e As ModificationExceptionEventArgs)
    Public Event ItemSelected(ByVal e As EventArgs)


    ''' <summary>
    ''' This class wraps the exception thrown into an EventArgs descendent that can be included
    ''' in a custom event. 
    ''' </summary>
    ''' <remarks>
    ''' This allows the code that includes the NavigationTree to simply catch the custom event
    ''' to handle exceptions that occur within.
    ''' </remarks>
    Public Class ModificationExceptionEventArgs
        Inherits EventArgs

        Private _InnerExceptionMessage As String

        Public Sub New(ByVal InnerExceptionMessage As String)
            Me.ExceptionMessage = InnerExceptionMessage
        End Sub

        Public Property ExceptionMessage()
            Get
                If (IsNothing(Me._InnerExceptionMessage)) Then Me._InnerExceptionMessage = ""
                If (Me._InnerExceptionMessage.Length = 0) Then Me._InnerExceptionMessage = "{no detail available}"
                Return Me._InnerExceptionMessage.Trim
            End Get
            Set(ByVal value)
                If (IsNothing(value)) Then value = ""
                Me._InnerExceptionMessage = value.trim
            End Set
        End Property
    End Class
#End Region



    Public Sub New()
        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.
        Me.TitleLabel.Text = ""
        Me.MainTreeView.LabelEdit = True
    End Sub


    Public Delegate Sub AddGroup()
    Public Delegate Sub AddNode()

    Private _AddGroup As AddGroup
    Private _AddNode As AddNode


#Region " Properties "
    ''' <summary>
    ''' This exposes the control's title, which sits above the treeview on the left side.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Title() As String
        Get
            Return Me.TitleLabel.Text.Trim
        End Get
        Set(ByVal value As String)
            If (IsNothing(value)) Then value = ""
            Me.TitleLabel.Text = value.Trim
        End Set
    End Property


    ''' <summary>
    ''' This exposes the Nodes collection, providing a slight abstraction for the underlying treeview structure.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public ReadOnly Property Items() As TreeNodeCollection
        Get
            Return Me.MainTreeView.Nodes
        End Get
    End Property


    ''' <summary>
    ''' This returns the selected SiteNode node if the user has one selected.  If the user has a group selected, this returns Nothing.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property SelectedItem() As NavigationTreeNode
        Get
            If (IsNothing(Me.MainTreeView.SelectedNode)) Then Return Nothing
            Return CType(Me.MainTreeView.SelectedNode, NavigationTreeNode)
        End Get
        Set(ByVal value As NavigationTreeNode)
            Me.MainTreeView.SelectedNode = value
        End Set
    End Property


    ''' <summary>
    ''' This returns True if the user has a SiteNode selected, and False otherwise.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property IsItemSelected() As Boolean
        Get
            If (IsNothing(Me.MainTreeView.SelectedNode)) Then Return False
            Return (Me.MainTreeView.SelectedNode.Level = 1)
        End Get
    End Property


    ''' <summary>
    ''' This returns True if the user has a group selected, and False otherwise.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private ReadOnly Property IsItemGroupSelected() As Boolean
        Get
            If (IsNothing(Me.MainTreeView.SelectedNode)) Then Return False
            Return (Me.MainTreeView.SelectedNode.Level = 0)
        End Get
    End Property


    ''' <summary>
    ''' This is used to set/access a reference to the method to be used to add a new group.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property AddGroupMethod() As AddGroup
        Get
            Return Me._AddGroup
        End Get
        Set(ByVal value As AddGroup)
            Me._AddGroup = value
        End Set
    End Property


    ''' <summary>
    ''' This is used to set/access a reference to the method to be used to add a new script.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property AddNodeMethod() As AddNode
        Get
            Return Me._AddNode
        End Get
        Set(ByVal value As AddNode)
            Me._AddNode = value
        End Set
    End Property
#End Region


#Region " Methods "
    ''' <summary>
    ''' This exposes the tree view's ExpandAll method.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ExpandAll()
        Me.MainTreeView.ExpandAll()
    End Sub


    ''' <summary>
    ''' This forces the top-most node in the tree to be visible.
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ScrollToTop()
        If (IsNothing(Me.MainTreeView)) Then Return
        If (Me.MainTreeView.Nodes.Count = 0) Then Return
        Me.MainTreeView.TopNode = Me.MainTreeView.Nodes(0)
    End Sub


    ''' <summary>
    ''' This will prompt the user to confirm deletion of the selected item; if they confirm it will remove it, and raise the ItemsEdited event.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RemoveNode()
        Dim CurrentNodeText As String

        CurrentNodeText = Me.MainTreeView.SelectedNode.Text
        If (Me.IsItemSelected) Then
            If (MsgBox("Do you really want to remove '" & CurrentNodeText & "'?", MsgBoxStyle.YesNo) = MsgBoxResult.No) Then Return
        Else
            If (MsgBox("This will remove the '" & CurrentNodeText & "' group, and all items contained in it.  Do you really want to do this?", MsgBoxStyle.YesNo) = MsgBoxResult.No) Then Return
        End If

        Me.MainTreeView.SelectedNode.Remove()
        RaiseEvent ItemsEdited(New EventArgs)
    End Sub


    ''' <summary>
    ''' This will begin editing of the selected tree node.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub RenameNode()
        If (Not IsNothing(Me.MainTreeView.SelectedNode)) Then Me.MainTreeView.SelectedNode.BeginEdit()
    End Sub


    ''' <summary>
    ''' This will refresh the UI controls when a new node in the tree has been selected.
    ''' </summary>
    ''' <remarks></remarks>
    Private Sub UpdateUIWhenNodeSelected()
        ' Tree buttons
        Me.NewFolderButton.Enabled = True
        Me.NewItemButton.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.ExecuteButton.Enabled = Me.IsItemSelected
        Me.RenameButton.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.DeleteButton.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected

        ' Context Menu buttons
        Me.AddGroupToolStripMenuItem.Enabled = True
        Me.AddItemToolStripMenuItem.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.RunToolStripMenuItem.Enabled = Me.IsItemSelected
        Me.RenameToolStripMenuItem.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected
        Me.DeleteToolStripMenuItem.Enabled = Me.IsItemSelected Or Me.IsItemGroupSelected

        RaiseEvent ItemSelected(New EventArgs)
    End Sub
#End Region


#Region " Event Handlers "
    Private Sub NavigationTree_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Me.UpdateUIWhenNodeSelected()
    End Sub


    Private Sub MainTreeView_AfterLabelEdit(ByVal sender As Object, ByVal e As System.Windows.Forms.NodeLabelEditEventArgs) Handles MainTreeView.AfterLabelEdit
        Try
            If (IsNothing(e.Label)) Then Return

            CType(Me.MainTreeView.SelectedNode, NavigationTreeNode).DisplayName = e.Label
            e.CancelEdit = False
            RaiseEvent ItemsEdited(e)
        Catch ex As Exception
            RaiseEvent ModificationExceptionThrown(New ModificationExceptionEventArgs("An error occurred completing your last modification: " & ex.Message))
            e.CancelEdit = True
        End Try
    End Sub


    Private Sub MainTreeView_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles MainTreeView.AfterSelect
        Me.UpdateUIWhenNodeSelected()
    End Sub


    Private Sub MainTreeView_DoubleClick(ByVal sender As Object, ByVal e As System.EventArgs) Handles MainTreeView.DoubleClick
        If (Me.IsItemSelected) Then RaiseEvent ItemExecuted(New EventArgs)
    End Sub


    Private Sub ExecuteButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExecuteButton.Click
        If (Me.IsItemSelected) Then RaiseEvent ItemExecuted(New EventArgs)
    End Sub


    Private Sub NewFolderButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NewFolderButton.Click
        If (Me._AddGroup Is Nothing) Then Throw New Exception("Delegate for the AddGroup method not defined.")
        Me._AddGroup()
    End Sub


    Private Sub DeleteButton_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles DeleteButton.Click
        Me.RemoveNode()
        Me.UpdateUIWhenNodeSelected()
    End Sub


    Private Sub NewItemButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles NewItemButton.Click
        If (Me._AddNode Is Nothing) Then Throw New Exception("Delegate for the AddNode method not defined.")
        Me._AddNode()
    End Sub


    Private Sub RenameButton_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RenameButton.Click
        Me.RenameNode()
    End Sub


    Private Sub AddGroupToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddGroupToolStripMenuItem.Click
        If (Me._AddGroup Is Nothing) Then Throw New Exception("Delegate for the AddGroup method not defined.")
        Me._AddGroup()
    End Sub


    Private Sub AddItemToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AddItemToolStripMenuItem.Click
        If (Me._AddNode Is Nothing) Then Throw New Exception("Delegate for the AddNode method not defined.")
        Me._AddNode()
    End Sub


    Private Sub RunToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RunToolStripMenuItem.Click
        RaiseEvent ItemExecuted(New EventArgs)
    End Sub


    Private Sub RenameToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles RenameToolStripMenuItem.Click
        Me.RenameNode()
    End Sub


    Private Sub DeleteToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DeleteToolStripMenuItem.Click
        Me.RemoveNode()
        Me.UpdateUIWhenNodeSelected()
    End Sub


    ''' <summary>
    ''' This looks for and routes the F2 keystroke (rename), Delete keystroke (remove item), and Enter keystroke (execute item)
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainTreeView_KeyDown(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MainTreeView.KeyDown
        If (e.KeyCode = Keys.F2) Then
            Me.RenameNode()
        ElseIf (e.KeyCode = Keys.Delete) Then
            Me.RemoveNode()
            Me.UpdateUIWhenNodeSelected()
        ElseIf (e.KeyCode = Keys.Enter) Then
            If (Me.IsItemSelected) Then RaiseEvent ItemExecuted(New EventArgs)
        End If
    End Sub


    ''' <summary>
    ''' This forces the node that was clicked to be the selected node.  The two would be out of sync if the user
    ''' had one selected, and then right-clicked another.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Private Sub MainTreeView_NodeMouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeNodeMouseClickEventArgs) Handles MainTreeView.NodeMouseClick
        If (IsNothing(CType(sender, TreeView).SelectedNode)) Then CType(sender, TreeView).SelectedNode = e.Node
        If (Not CType(sender, TreeView).SelectedNode.Equals(e.Node)) Then Me.MainTreeView.SelectedNode = e.Node
    End Sub


    ' The next three handlers cover the drag-and-drop functionality.  Only Script nodes can be dragged at this time.
    Private Sub MainTreeView_ItemDrag(ByVal sender As Object, ByVal e As System.Windows.Forms.ItemDragEventArgs) Handles MainTreeView.ItemDrag
        If (CType(e.Item, NavigationTreeNode).Level = 1) Then
            Me.DoDragDrop(e.Item, DragDropEffects.Move)
        Else
            Me.DoDragDrop(e.Item, DragDropEffects.None)
        End If
    End Sub
    Private Sub MainTreeView_DragEnter(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MainTreeView.DragEnter
        e.Effect = DragDropEffects.Move
    End Sub
    Private Sub MainTreeView_DragDrop(ByVal sender As Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MainTreeView.DragDrop
        Dim NodeBeingMoved, OldParentNode, ClonedNode As NavigationTreeNode

        If (e.Data.GetDataPresent("NAntRunner.NavigationTreeNode", False)) Then
            Dim pt As Point = CType(sender, TreeView).PointToClient(New Point(e.X, e.Y))
            Dim DestinationNode As NavigationTreeNode = CType(sender, TreeView).GetNodeAt(pt)
            NodeBeingMoved = CType(e.Data.GetData("NAntRunner.NavigationTreeNode"), NavigationTreeNode)

            If (NodeBeingMoved.Level = 0) Then Return
            OldParentNode = NodeBeingMoved.Parent

            ' Clone the node that is moving so we can add it to another group if need be
            ClonedNode = NodeBeingMoved.Clone
            ClonedNode.Path = NodeBeingMoved.Path
            ClonedNode.DisplayName = NodeBeingMoved.DisplayName

            If (DestinationNode.Level = 0) Then
                ' Add to the end of the group that it was dropped on.
                DestinationNode.Nodes.Add(ClonedNode)
                DestinationNode.Expand()
            Else
                ' Add to the slot just above the DestinationNode.
                DestinationNode.Parent.Nodes.Insert(DestinationNode.Index, ClonedNode)
                DestinationNode.Parent.Expand()
            End If
            NodeBeingMoved.Remove()

            RaiseEvent ItemsEdited(New EventArgs)
        End If
    End Sub

#End Region
End Class
