﻿' *****************************************************************************
' NAntRunner
' A simple, standalone, utility for running NAnt build scripts.
'
' Developed by Mark Gilbert (http://www.mark-gilbert.com).
'
' This software is license under the Microsoft Public License (Ms-PL).  For full
' license details, please see 
' http://www.microsoft.com/resources/sharedsource/licensingbasics/publiclicense.mspx.
' *****************************************************************************
Imports NUnit.Framework
Imports NAntRunner
Imports System.Text.RegularExpressions
Imports NRBusinessServices

<TestFixture()> _
Public Class NAntProcessTests

    Private _MyNAntProcess As NAntProcess

    Private Const PATH_TO_NANT As String = "C:\Some\Path\nant.exe"
    Private Const PATH_TO_SCRIPT As String = "C:\Some\Path\MyScript.build"

#Region " Executable Tests "
    <Test()> _
    Public Sub InstantiateClassWithParameters_NAntExecutablePathAvailable()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT)
        Assert.AreEqual(PATH_TO_NANT, Me._MyNAntProcess.NAntExecutable)
    End Sub

    <Test()> _
    Public Sub InstantiateClassWithNothingExe_ExceptionThrown()
        Dim WasExceptionThrown, WasCorrectExceptionThrown As Boolean
        Dim ExceptionMessage As String

        WasCorrectExceptionThrown = False
        WasExceptionThrown = False
        ExceptionMessage = ""
        Try
            Me._MyNAntProcess = New NAntProcess(Nothing, PATH_TO_SCRIPT)
        Catch ex As NAntProcess.ExecutableMissingException
            WasExceptionThrown = True
            WasCorrectExceptionThrown = True
        Catch ex As Exception
            WasExceptionThrown = True
            WasCorrectExceptionThrown = False
            ExceptionMessage = ex.Message
        End Try

        If (Not WasExceptionThrown) Then Assert.Fail("No exception thrown")
        If (Not WasCorrectExceptionThrown) Then Assert.Fail("Incorrect exception was thrown.  Message: " & ExceptionMessage)
    End Sub

    <Test()> _
    Public Sub InstantiateClassWithEmptyExe_ExceptionThrown()
        Dim WasExceptionThrown, WasCorrectExceptionThrown As Boolean
        Dim ExceptionMessage As String

        WasCorrectExceptionThrown = False
        WasExceptionThrown = False
        ExceptionMessage = ""
        Try
            Me._MyNAntProcess = New NAntProcess("", PATH_TO_SCRIPT)
        Catch ex As NAntProcess.ExecutableMissingException
            WasExceptionThrown = True
            WasCorrectExceptionThrown = True
        Catch ex As Exception
            WasExceptionThrown = True
            WasCorrectExceptionThrown = False
            ExceptionMessage = ex.Message
        End Try

        If (Not WasExceptionThrown) Then Assert.Fail("No exception thrown")
        If (Not WasCorrectExceptionThrown) Then Assert.Fail("Incorrect exception was thrown.  Message: " & ExceptionMessage)
    End Sub

#End Region


#Region " Script Path Tests "
    <Test()> _
    Public Sub InstantiateClassWithParameters_ScriptPathAvailable()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT)
        Assert.AreEqual(PATH_TO_SCRIPT, Me._MyNAntProcess.ScriptToRun)
    End Sub
    <Test()> _
    Public Sub InstantiateClassWithNothingScript_ExceptionThrown()
        Dim WasExceptionThrown, WasCorrectExceptionThrown As Boolean
        Dim ExceptionMessage As String

        WasCorrectExceptionThrown = False
        WasExceptionThrown = False
        ExceptionMessage = ""
        Try
            Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, Nothing)
        Catch ex As NAntProcess.ScriptToRunMissingException
            WasExceptionThrown = True
            WasCorrectExceptionThrown = True
        Catch ex As Exception
            WasExceptionThrown = True
            WasCorrectExceptionThrown = False
            ExceptionMessage = ex.Message
        End Try

        If (Not WasExceptionThrown) Then Assert.Fail("No exception thrown")
        If (Not WasCorrectExceptionThrown) Then Assert.Fail("Incorrect exception was thrown.  Message: " & ExceptionMessage)
    End Sub

    <Test()> _
    Public Sub InstantiateClassWithEmptyScript_ExceptionThrown()
        Dim WasExceptionThrown, WasCorrectExceptionThrown As Boolean
        Dim ExceptionMessage As String

        WasCorrectExceptionThrown = False
        WasExceptionThrown = False
        ExceptionMessage = ""
        Try
            Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, "")
        Catch ex As NAntProcess.ScriptToRunMissingException
            WasExceptionThrown = True
            WasCorrectExceptionThrown = True
        Catch ex As Exception
            WasExceptionThrown = True
            WasCorrectExceptionThrown = False
            ExceptionMessage = ex.Message
        End Try

        If (Not WasExceptionThrown) Then Assert.Fail("No exception thrown")
        If (Not WasCorrectExceptionThrown) Then Assert.Fail("Incorrect exception was thrown.  Message: " & ExceptionMessage)
    End Sub

    <Test()> _
    Public Sub SpecifyBuildScript_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT)
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-buildfile:""" & PATH_TO_SCRIPT & """"))
    End Sub
#End Region


#Region " Target Framework Tests "
    <Test()> _
    Public Sub TargetFrameworkNotSpecified_NotIncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "")
        Assert.IsFalse(Me._MyNAntProcess.BuildArguments.Contains("-targetframework:"))
    End Sub

    <Test()> _
    Public Sub SpecifyTargetFrameworkAsNet1_0_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "net-1.0")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-targetframework:net-1.0"))
    End Sub

    <Test()> _
    Public Sub SpecifyTargetFrameworkAsNet1_1_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "net-1.1")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-targetframework:net-1.1"))
    End Sub

    <Test()> _
    Public Sub SpecifyTargetFrameworkAsNet2_0_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "net-2.0")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-targetframework:net-2.0"))
    End Sub
#End Region


#Region " Build Target Tests "
    <Test()> _
    Public Sub SpecifyBuildTargetA_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "", "Custom", False, "")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains(" Custom"), Me._MyNAntProcess.BuildArguments)
    End Sub

    <Test()> _
    Public Sub SpecifyBuildTargetB_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "", "Custom2", False, "")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains(" Custom2"), Me._MyNAntProcess.BuildArguments)
    End Sub
#End Region


#Region " EnableVerboseMessages Tests "
    <Test()> _
    Public Sub EnableVerboseMessages_NotChecked_NotIncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "", "Custom2", False, "")
        Assert.IsFalse(Me._MyNAntProcess.BuildArguments.Contains(" -verbose"), Me._MyNAntProcess.BuildArguments)
    End Sub
    <Test()> _
    Public Sub EnableVerboseMessages_Checked_NotIncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "", "Custom2", True, "")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains(" -verbose"), Me._MyNAntProcess.BuildArguments)
    End Sub
#End Region



#Region " OtherArgs Tests "
    <Test()> _
    Public Sub SpecifyOtherArgsA_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "", "", False, "-help")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-help"))
    End Sub

    <Test()> _
    Public Sub SpecifyOtherArgsB_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "", "", False, "-blah:""Value and another value""")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-blah:""Value and another value"""))
    End Sub
#End Region


#Region " Mixed Parameter Tests"

    <Test()> _
    Public Sub SpecifyMultipleParametersA_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "net-1.0", "", False, "-blah:""Value and another value"": " & vbCrLf & Me._MyNAntProcess.BuildArguments)
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-blah:""Value and another value"""), "'Other' build parameter missing: " & vbCrLf & Me._MyNAntProcess.BuildArguments)
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-targetframework:net-1.0"), "Target framework missing: " & vbCrLf & Me._MyNAntProcess.BuildArguments)
    End Sub

    <Test()> _
    Public Sub SpecifyMultipleParametersB_IncludedInBuildArguments()
        Me._MyNAntProcess = New NAntProcess(PATH_TO_NANT, PATH_TO_SCRIPT, "net-2.0", "sweet", False, "-blah:just_another_param")
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-blah:just_another_param"), "'Other' build parameter missing: " & vbCrLf & Me._MyNAntProcess.BuildArguments)
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("-targetframework:net-2.0"), "Target framework missing: " & vbCrLf & Me._MyNAntProcess.BuildArguments)
        Assert.IsTrue(Me._MyNAntProcess.BuildArguments.Contains("sweet"), "Build target missing: " & vbCrLf & Me._MyNAntProcess.BuildArguments)
    End Sub
#End Region

End Class
